@@ -20,7 +20,7 @@
 # 
 #----------------------------------------------------------------------------
 #
-# $Id: Document.pm,v 2.49 2002/04/17 14:04:38 abw Exp $
+# $Id: Document.pm,v 2.56 2002/07/30 12:44:56 abw Exp $
 #
 #============================================================================
 
@@ -32,9 +32,11 @@ use strict;
 use vars qw( $VERSION $ERROR $COMPERR $DEBUG $AUTOLOAD );
 use base qw( Template::Base );
 use Template::Constants;
-use Fcntl qw(O_WRONLY O_CREAT O_TRUNC);
+#use Fcntl qw(O_WRONLY O_CREAT O_TRUNC);
+use File::Temp qw( tempfile );
+use File::Basename;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.49 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.56 $ =~ /(\d+)\.(\d+)/);
 
 
 #========================================================================
@@ -200,76 +202,90 @@ sub _dump {
 
 
 #========================================================================
-#                      ----- PACKAGE SUBS -----
+#                      ----- CLASS METHODS -----
 #========================================================================
 
 #------------------------------------------------------------------------
-# write_perl_file($filename, \%content)
+# as_perl($content)
 #
-# This sub-routine writes the Perl code representing a compiled
-# template to a file, specified by name as the first parameter.
-# The second parameter should be a hash array containing a main
-# template BLOCK, a hash array of additional DEFBLOCKS (named BLOCKs
-# definined in the template document source) and a hash array of
-# METADATA items.  The values for the BLOCK and individual BLOCKS
-# entries should be strings containing Perl code representing the
-# templates as compiled by the parser.
-#
-# Returns 1 on success.  On error, sets the $ERROR package variable
-# to contain an error message and returns undef.
-#
-# This is a bit of an ugly hack.  It might be better if the Document
-# object itself had an as_perl() method to return a Perl representation
-# of itself.  But that would imply it had to store it's Perl text 
-# as well as a reference to the evaluated Perl sub-routines.  Using this
-# approach, we can let the new() constructor eval() the Perl code
-# and then discard the source text.
+# This method expects a reference to a hash passed as the first argument
+# containing 3 items:
+#     METADATA   # a hash of template metadata
+#     BLOCK      # string containing Perl sub definition for main block
+#     DEFBLOCKS  # hash containing further subs for addional BLOCK defs
+# It returns a string containing Perl code which, when evaluated and 
+# executed, will instantiate a new Template::Document object with the 
+# above data.  On error, it returns undef with an appropriate error
+# message set in $ERROR.
 #------------------------------------------------------------------------
 
-sub write_perl_file {
-    my ($file, $content) = @_;
-    my ($block, $defblocks, $metadata) = 
-	@$content{ qw( BLOCK DEFBLOCKS METADATA ) };
-    my $pkg = __PACKAGE__;
+sub as_perl {
+    my ($class, $content) = @_;
+    my ($block, $defblocks, $metadata) = @$content{ qw( BLOCK DEFBLOCKS METADATA ) };
 
-    $defblocks = join('', 
-		      map { "'$_' => $defblocks->{ $_ },\n" }
-		      keys %$defblocks);
+    $block =~ s/\n/\n    /g;
+    $block =~ s/\s+$//;
 
-    $metadata = join('', 
-		       map { 
-			   my $x = $metadata->{ $_ }; 
-			   $x =~ s/(['\\])/\\$1/g; 
-			   "'$_' => '$x',\n";
-		       } keys %$metadata);
+    $defblocks = join('', map {
+	my $code = $defblocks->{ $_ };
+	$code =~ s/\n/\n        /g;
+	$code =~ s/\s*$//;
+	"        '$_' => $code,\n";
+    } keys %$defblocks);
+    $defblocks =~ s/\s+$//;
 
-    local *CFH;
-    my ($cfile) = $file =~ /^(.+)$/s or do {
-	$ERROR = "invalid filename: $file";
-	return undef;
-    };
-    sysopen(CFH, $cfile, O_CREAT|O_WRONLY|O_TRUNC) or do {
-	$ERROR = $!;
-	return undef;
-    };
+    $metadata = join('', map { 
+	my $x = $metadata->{ $_ }; 
+	$x =~ s/(['\\])/\\$1/g; 
+	"        '$_' => '$x',\n";
+    } keys %$metadata);
+    $metadata =~ s/\s+$//;
 
-    print CFH  <<EOF;
+    return <<EOF
 #------------------------------------------------------------------------
 # Compiled template generated by the Template Toolkit version $Template::VERSION
 #------------------------------------------------------------------------
 
-bless {
+$class->new({
+    METADATA => {
 $metadata
-_HOT       => 0,
-_BLOCK     => $block,
-_DEFBLOCKS => {
+    },
+    BLOCK => $block,
+    DEFBLOCKS => {
 $defblocks
-},
-}, $pkg;
+    },
+});
 EOF
-    close(CFH);
+}
 
-    return 1;
+
+#------------------------------------------------------------------------
+# write_perl_file($filename, \%content)
+#
+# This method calls as_perl() to generate the Perl code to represent a
+# compiled template with the content passed as the second argument.
+# It then writes this to the file denoted by the first argument.
+#
+# Returns 1 on success.  On error, sets the $ERROR package variable
+# to contain an error message and returns undef.
+#------------------------------------------------------------------------
+
+sub write_perl_file {
+    my ($class, $file, $content) = @_;
+    my ($fh, $tmpfile);
+    
+    return $class->error("invalid filename: $file")
+	unless $file =~ /^(.+)$/s;
+
+    eval {
+	($fh, $tmpfile) = tempfile( DIR => dirname($file) );
+	print $fh $class->as_perl($content) || die $!;
+	close($fh);
+    };
+    return $class->error($@) if $@;
+
+    return rename($tmpfile, $file)
+	|| $class->error($!);
 }
 
 
@@ -441,7 +457,7 @@ have persistance of compiled templates.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@kfs.orgE<gt>
+Andy Wardley E<lt>abw@andywardley.comE<gt>
 
 L<http://www.andywardley.com/|http://www.andywardley.com/>
 
@@ -450,8 +466,8 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.49, distributed as part of the
-Template Toolkit version 2.07, released on 17 April 2002.
+2.55, distributed as part of the
+Template Toolkit version 2.08, released on 30 July 2002.
 
 =head1 COPYRIGHT
 
